// Generated from tmpfs-options.json. Do not edit!
#ifndef _GNU_SOURCE
#define _GNU_SOURCE
#endif
#include <string.h>
#include <read_file.h>
#include "tmpfs_options.h"

tmpfs_options *make_tmpfs_options(yajl_val tree, const struct parser_context *ctx, parser_error *err) {
    tmpfs_options *ret = NULL;
    *err = 0;
    if (tree == NULL)
        return ret;
    ret = safe_malloc(sizeof(*ret));
    {
        yajl_val val = get_val(tree, "SizeBytes", yajl_t_number);
        if (val != NULL) {
            int invalid = common_safe_int64(YAJL_GET_NUMBER(val), &ret->size_bytes);
            if (invalid) {
                if (asprintf(err, "Invalid value '%s' with type 'int64' for key 'SizeBytes': %s", YAJL_GET_NUMBER(val), strerror(-invalid)) < 0)
                    *err = safe_strdup("error allocating memory");
                free_tmpfs_options(ret);
                return NULL;
            }
        }
    }
    {
        yajl_val val = get_val(tree, "Mode", yajl_t_number);
        if (val != NULL) {
            int invalid = common_safe_uint32(YAJL_GET_NUMBER(val), &ret->mode);
            if (invalid) {
                if (asprintf(err, "Invalid value '%s' with type 'uint32' for key 'Mode': %s", YAJL_GET_NUMBER(val), strerror(-invalid)) < 0)
                    *err = safe_strdup("error allocating memory");
                free_tmpfs_options(ret);
                return NULL;
            }
        }
    }

    if (tree->type == yajl_t_object)
    {
        size_t i;
        size_t j = 0;
        size_t cnt = tree->u.object.len;
        yajl_val resi = NULL;

        if (ctx->options & OPT_PARSE_FULLKEY)
        {
            resi = calloc (1, sizeof(*tree));
            if (resi == NULL)
            {
                free_tmpfs_options(ret);
                return NULL;
            }
            resi->type = yajl_t_object;
            resi->u.object.keys = calloc (cnt, sizeof (const char *));
            if (resi->u.object.keys == NULL)
            {
               free_tmpfs_options(ret);
               yajl_tree_free(resi);
               return NULL;
            }
            resi->u.object.values = calloc (cnt, sizeof (yajl_val));
            if (resi->u.object.values == NULL)
            {
               free_tmpfs_options(ret);
               yajl_tree_free(resi);
               return NULL;
            }
        }
        for (i = 0; i < tree->u.object.len; i++)
        {
           if (strcmp(tree->u.object.keys[i], "SizeBytes") &&
                strcmp(tree->u.object.keys[i], "Mode"))
           {
              if (ctx->options & OPT_PARSE_FULLKEY)
              {
                 resi->u.object.keys[j] = tree->u.object.keys[i];
                 tree->u.object.keys[i] = NULL;
                 resi->u.object.values[j] = tree->u.object.values[i];
                 tree->u.object.values[i] = NULL;
                 resi->u.object.len++;
              }
              j++;
           }
        }
        if (ctx->options & OPT_PARSE_STRICT)
        {
            if (j > 0 && ctx->stderr > 0)
                fprintf (ctx->stderr, "WARNING: unknown key found\n");
        }
        if (ctx->options & OPT_PARSE_FULLKEY)
            ret->_residual = resi;
        }
    return ret;
}

void free_tmpfs_options(tmpfs_options *ptr) {
    if (ptr == NULL)
        return;
    yajl_tree_free (ptr->_residual);
    ptr->_residual = NULL;
    free(ptr);
}

yajl_gen_status gen_tmpfs_options(yajl_gen g, const tmpfs_options *ptr, const struct parser_context *ctx, parser_error *err) {
    yajl_gen_status stat = yajl_gen_status_ok;
    *err = 0;
    stat = yajl_gen_map_open((yajl_gen)g);
    if (yajl_gen_status_ok != stat)
        GEN_SET_ERROR_AND_RETURN(stat, err);
    if ((ctx->options & OPT_GEN_KAY_VALUE) || (ptr != NULL && ptr->size_bytes)) {
        long long int num = 0;
        stat = yajl_gen_string((yajl_gen)g,             (const unsigned char *)("SizeBytes"), strlen("SizeBytes"));
        if (yajl_gen_status_ok != stat)
            GEN_SET_ERROR_AND_RETURN(stat, err);
        if (ptr != NULL && ptr->size_bytes) {
            num = (long long int)ptr->size_bytes;
        }
        stat = map_int(g, num);
        if (yajl_gen_status_ok != stat)
            GEN_SET_ERROR_AND_RETURN(stat, err);
    }
    if ((ctx->options & OPT_GEN_KAY_VALUE) || (ptr != NULL && ptr->mode)) {
        long long unsigned int num = 0;
        stat = yajl_gen_string((yajl_gen)g,             (const unsigned char *)("Mode"), strlen("Mode"));
        if (yajl_gen_status_ok != stat)
            GEN_SET_ERROR_AND_RETURN(stat, err);
        if (ptr != NULL && ptr->mode) {
            num = (long long unsigned int)ptr->mode;
        }
        stat = map_uint(g, num);
        if (yajl_gen_status_ok != stat)
            GEN_SET_ERROR_AND_RETURN(stat, err);
    }
    if (ptr != NULL && ptr->_residual != NULL)
    {
        stat = gen_yajl_object_residual (ptr->_residual, g, err);
        if (yajl_gen_status_ok != stat)
            GEN_SET_ERROR_AND_RETURN (stat, err);
    }
    stat = yajl_gen_map_close((yajl_gen)g);
    if (yajl_gen_status_ok != stat)
        GEN_SET_ERROR_AND_RETURN(stat, err);
    return yajl_gen_status_ok;
}


tmpfs_options *tmpfs_options_parse_file(const char *filename, const struct parser_context *ctx, parser_error *err) {
    tmpfs_options *ptr = NULL;
    size_t filesize;
    char *content = NULL;

    if (filename == NULL || err == NULL)
        return NULL;

    *err = NULL;
    content = read_file(filename, &filesize);
    if (content == NULL) {
        if (asprintf(err, "cannot read the file: %s", filename) < 0)
            *err = safe_strdup("error allocating memory");
        return NULL;
    }
    ptr = tmpfs_options_parse_data(content, ctx, err);
    free(content);
    return ptr;
}

tmpfs_options *tmpfs_options_parse_file_stream(FILE *stream, const struct parser_context *ctx, parser_error *err) {
    tmpfs_options *ptr = NULL;
    size_t filesize;
    char *content = NULL ;

    if (stream == NULL || err == NULL)
        return NULL;

    *err = NULL;
    content = fread_file(stream, &filesize);
    if (content == NULL) {
        *err = safe_strdup("cannot read the file");
        return NULL;
    }
    ptr = tmpfs_options_parse_data(content, ctx, err);
    free(content);
    return ptr;
}

tmpfs_options *tmpfs_options_parse_data(const char *jsondata, const struct parser_context *ctx, parser_error *err) {
    tmpfs_options *ptr = NULL;
    yajl_val tree;
    char errbuf[1024];
    struct parser_context tmp_ctx = { 0 };

    if (jsondata == NULL || err == NULL)
        return NULL;

    *err = NULL;
    if (ctx == NULL) {
       ctx = (const struct parser_context *)(&tmp_ctx);
    }
    tree = yajl_tree_parse(jsondata, errbuf, sizeof(errbuf));
    if (tree == NULL) {
        if (asprintf(err, "cannot parse the data: %s", errbuf) < 0)
            *err = safe_strdup("error allocating memory");
        return NULL;
    }
    ptr = make_tmpfs_options(tree, ctx, err);
    yajl_tree_free(tree);
    return ptr;
}
char *tmpfs_options_generate_json(const tmpfs_options *ptr, const struct parser_context *ctx, parser_error *err) {
    yajl_gen g = NULL;
    struct parser_context tmp_ctx = { 0 };
    const unsigned char *gen_buf = NULL;
    char *json_buf = NULL;
    size_t gen_len = 0;

    if (ptr == NULL || err == NULL)
        return NULL;

    *err = NULL;
    if (ctx == NULL) {
        ctx = (const struct parser_context *)(&tmp_ctx);
    }

    if (!json_gen_init(&g, ctx)) {
        *err = safe_strdup("Json_gen init failed");
        goto out;
    }
    if (yajl_gen_status_ok != gen_tmpfs_options(g, ptr, ctx, err)) {
        if (*err == NULL)
            *err = safe_strdup("Failed to generate json");
        goto free_out;
    }
    yajl_gen_get_buf(g, &gen_buf, &gen_len);
    if (gen_buf == NULL) {
        *err = safe_strdup("Error to get generated json");
        goto free_out;
    }

    json_buf = safe_malloc(gen_len + 1);
    (void)memcpy(json_buf, gen_buf, gen_len);
    json_buf[gen_len] = '\0';

free_out:
    yajl_gen_clear(g);
    yajl_gen_free(g);
out:
    return json_buf;
}

